package com.winhc.dataworks.flow.touch;

import cn.hutool.cache.CacheUtil;
import cn.hutool.cache.impl.TimedCache;
import com.aliyuncs.dataworks_public.model.v20180601.CreateManualDagResponse;
import com.helospark.lightdi.LightDi;
import com.helospark.lightdi.LightDiContext;
import com.helospark.lightdi.annotation.Autowired;
import com.helospark.lightdi.annotation.Service;
import com.winhc.dataworks.flow.touch.bean.*;
import com.winhc.dataworks.flow.touch.configuration.DataWorksAccessProperties;
import com.winhc.dataworks.flow.touch.configuration.SchemaInit;
import com.winhc.dataworks.flow.touch.service.OdpsService;
import com.winhc.dataworks.flow.touch.service.SendJobStatus;
import com.winhc.dataworks.flow.touch.service.TouchService;
import com.winhc.dataworks.flow.touch.utils.DateUtils;
import com.winhc.dataworks.flow.touch.utils.DingUtils;
import com.winhc.dataworks.flow.touch.utils.SparkDaemonThread;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.cli.*;

import java.io.FileNotFoundException;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author: XuJiakai
 * @Date: 2020/6/22 10:45
 * @Description:
 */
@Slf4j
@Service
public class Main {
    @Autowired
    private TouchService touchService;

    @Autowired
    private DataWorksAccessProperties dataWorksAccessProperties;

    private static final Options options = new Options();


    @Autowired
    private DingUtils dingUtils;

    static {
        options.addOption("q", "query", false, "选填，查询昨天company增量表是否有数据");
        options.addOption("d", "debug", false, "选填，启用debug模式，只输出任务参数不进行调度");
        options.addOption("w", "wait", false, "选填，等待上游数据同步完成");


        options.addOption("s", "singleJob", false, "选填，是否单独触发一个业务流程");
        options.addOption("b", "bizDate", true, "选填，业务时间[2020-07-07],默认为昨天");
        options.addOption("f", "fileName", true, "必填，yaml文件");

        options.addOption("flow", "flow", true, "单任务必填，业务流程名");
        options.addOption("task", "taskName", true, "单任务必填，任务名");
        options.addOption("odps", "odps_home", true, "odps cmd 根目录");

    }

    private static void verify(CommandLine commandLine) {
        try {
            if (!commandLine.hasOption("f")) {
                throw new RuntimeException();
            }
            if (!commandLine.hasOption("odps")) {
                throw new RuntimeException();
            }
            if (commandLine.hasOption("s")) {
                if (!commandLine.hasOption("flow") || !commandLine.hasOption("task")) {
                    throw new RuntimeException();
                }
            }
        } catch (Exception e) {
            HelpFormatter formatter = new HelpFormatter();
            formatter.printHelp("很棒，参数错误", options);
            System.exit(-1);
        }
    }


    public static void main(String[] args) {
        LightDiContext context = LightDi.initContextByPackage(Main.class.getPackage().getName());
        DingUtils dd = context.getBean(DingUtils.class);
        CommandLineParser parser = new DefaultParser();
        try {
            CommandLine commandLine = parser.parse(options, args);

            if (commandLine.hasOption("q")) {
                OdpsService bean = context.getBean(OdpsService.class);
                String ds = DateUtils.getYesterday().replace("-", "");

//                List<String> notEmptyTable = bean.getNotEmptyTable(ds);
//                HashSet<String> set = new HashSet<>(notEmptyTable);
                Boolean success = false;
                success = bean.companyTableIsNotEmpty(DateUtils.getYesterday());
                if (success) {
                    log.info("company表{}分区数据不为空.", ds);
                } else {
                    log.error("company表{}分区数据为空，调度程序中止!", ds);
                    dd.send("company表" + ds + "分区数据为空，调度程序中止!");
                    System.exit(-1);
                }
                return;
            }

            if (commandLine.hasOption("w")) {
                QueryStatus bean = context.getBean(QueryStatus.class);
                bean.waitTaskDone();
                return;
            }

            verify(commandLine);
            String bizDate = commandLine.getOptionValue("b", DateUtils.getYesterday());
            String fileName = commandLine.getOptionValue("f");
            List<DataWorksFlowJob> jobs = SchemaInit.getJobs(fileName);

            if (commandLine.hasOption("s")) {
                log.info("执行单个业务流程！");
                String flow = commandLine.getOptionValue("flow");
                String task = commandLine.getOptionValue("task");
                SingleJobMain singleJobMain = context.getBean(SingleJobMain.class);
                String msg = "开始执行单个业务流程！bizDate:" + bizDate + " flow:" + flow + " task:" + task + " JobFile:" + commandLine.getOptionValue("f");
                log.info(msg);
                if (commandLine.hasOption("d")) {
                    log.info("debug模式：{}", jobs);
                } else {
                    singleJobMain.start(flow, task, bizDate, jobs);
                }
            } else {
                log.info("触发全部业务流程！");
                Main bean = context.getBean(Main.class);
                String msg = "开始执行全部业务流程！bizDate:" + bizDate + " JobFile:" + commandLine.getOptionValue("f");
                log.info(msg);
                if (commandLine.hasOption("d")) {
                    log.info("debug模式：{}", jobs);
                } else {
                    String odpsCmdPath = commandLine.getOptionValue("odps");
                    dd.send(msg);
                    bean.start(bizDate, jobs, odpsCmdPath);
                    // 指定job结束后发送通知
                    if (fileName.endsWith("step03.yaml")) {
                        SendJobStatus sendJobStatus = context.getBean(SendJobStatus.class);
                        sendJobStatus.sendJobStatus();
                    }
                }
            }
        } catch (ParseException e) {
            HelpFormatter formatter = new HelpFormatter();
            formatter.printHelp("很棒，参数错误", options);
            System.exit(-1);
        } catch (FileNotFoundException e) {
            log.error(e.getMessage());
            System.exit(-2);
        } catch (Exception e) {
            dd.send(e.getMessage());
            log.error(e.getMessage(), e);
            System.exit(-3);
        }
    }

    @SneakyThrows
    private void start(String bizDate, List<DataWorksFlowJob> jobs, String odpsCmdHome) {
        //为所有项目空间启动守护线程
        String accessKeyId = dataWorksAccessProperties.getAccessKeyId();
        String accessKeySecret = dataWorksAccessProperties.getAccessKeySecret();
        Set<String> ps = jobs.stream().map(DataWorksFlowJob::getProject).collect(Collectors.toSet());
        for (String p : ps) {
            new SparkDaemonThread(p, accessKeyId, accessKeySecret, odpsCmdHome, 90L).start();
        }

        //运行job，并接收失败参数，最大重试三次
        Set<TaskInfo> failureTask = run(bizDate, jobs);
        int i = 3;
        while (!failureTask.isEmpty() && i-- > 0) {
            Set<String> fSet = failureTask.stream().map(TaskInfo::getKey).collect(Collectors.toSet());

            List<DataWorksFlowJob> js = jobs.stream().map(job -> {
                String project = job.getProject();
                String flow = job.getFlow();
                List<DataWorksFlowTask> task = job.getTask();

                List<DataWorksFlowTask> collect = task.stream().filter(t -> fSet.contains(project + ":" + flow + ":" + t.getTaskName())
                ).collect(Collectors.toList());

                if (collect.isEmpty()) {
                    return null;
                } else {
                    return new DataWorksFlowJob(project, flow, collect);
                }
            }).filter(Objects::nonNull).collect(Collectors.toList());

            String collect = js.stream().flatMap(job -> {
                String project = job.getProject();
                String flow = job.getFlow();
                List<DataWorksFlowTask> task = job.getTask();
                return task.stream().map(t -> project + ":" + flow + ":" + t.getTaskName());
            }).collect(Collectors.joining(","));
            dingUtils.send("【" + (3 - i) + "】重新启动以下job：" + collect);
            failureTask = run(bizDate, js);
        }
        if (!failureTask.isEmpty()) {
            System.exit(-1);
        }
    }

    @SneakyThrows
    private Set<TaskInfo> run(String bizDate, List<DataWorksFlowJob> jobs) {
        log.info("start!");
        LocalDateTime start = LocalDateTime.now();
        List<TaskInfo> collect = jobs.stream()
                .flatMap(dataWorksFlowJob -> {
                    List<DataWorksFlowTask> task = dataWorksFlowJob.getTask();
                    return task
                            .stream()
                            .map(dataWorksFlowTask -> {
                                TaskParam build = TaskParam.builder()
                                        .projectName(dataWorksFlowJob.getProject())
                                        .bizDate(bizDate)
                                        .flowName(dataWorksFlowJob.getFlow())
                                        .nodeParam(dataWorksFlowTask.toNodeParam(bizDate))
                                        .build();
                                CreateManualDagResponse r = touchService.touch(build);

                                return TaskInfo.builder().project(dataWorksFlowJob.getProject())
                                        .flow(dataWorksFlowJob.getFlow())
                                        .taskName(dataWorksFlowTask.getTaskName())
                                        .dagId(r.getReturnValue()).build();
                            });
                }).collect(Collectors.toList());

        int totalTask = collect.size();

        Set<TaskInfo> end = new HashSet<>();
        Set<TaskInfo> failureTask = new HashSet<>();
        TimedCache<TaskInfo, String> timedCache = CacheUtil.newTimedCache(300 * 1000);
        int i = 0;
        int successTask = 0;
        int failedTask = 0;
        while (true) {
            int awaitTask = 0;

            boolean flag = true;
            List<String> empty = new ArrayList<>();

            for (TaskInfo taskInfo : collect) {
                if (end.contains(taskInfo)) {
                    continue;
                }
                Map<String, TaskFlowEnum> query = touchService.query(taskInfo.getProject(), taskInfo.getDagId());
                Map<String, List<String>> status = query.entrySet().stream()
                        .map(e -> {
                            Entry<String, String> entry = new Entry<>();
                            if (e.getValue().equals(TaskFlowEnum.SUCCESS) || e.getValue().equals(TaskFlowEnum.FAILURE)) {
                                entry.setKey(e.getValue().getMsg());
                            } else {
                                entry.setKey("运行中");
                            }
                            entry.setValue(e.getKey());
                            return entry;
                        }).collect(Collectors.groupingBy(Entry::getKey, Collectors.mapping(Entry::getValue, Collectors.toList())));
                List<String> success = status.getOrDefault(TaskFlowEnum.SUCCESS.getMsg(), empty);
                List<String> failure = status.getOrDefault(TaskFlowEnum.FAILURE.getMsg(), empty);
                List<String> await = status.getOrDefault("运行中", empty);


                if (failure.size() != 0) {
                    failedTask++;
                    log.error("failure node:{} ", failure);
                    DingMsg error = new DingMsg("任务失败", taskInfo.getProject(), taskInfo.getFlow(), taskInfo.getTaskName(), String.join(",", failure), TaskFlowEnum.FAILURE.getMsg());
//                    dingUtils.send(error);
                    failureTask.add(taskInfo);
                } else {
                    if (await.size() != 0) {
                        awaitTask++;
                    } else {
                        successTask++;
                    }
                }


                flag = flag && (await.size() == 0 || failure.size() != 0);

                if (await.size() == 0 || failure.size() != 0) {
                    end.add(taskInfo);
                } else {
                    LocalDateTime now = LocalDateTime.now();
                    Duration duration = Duration.between(start, now);
                    long l = duration.toHours();
                    if (l >= 2) {
                        if (!timedCache.containsKey(taskInfo) && i <= 6) {
                            //超两小时
                            i++;
                            DingMsg error = new DingMsg("【" + i + "】任务长时间未结束", taskInfo.getProject(), taskInfo.getFlow(), taskInfo.getTaskName(), String.join(",", failure), TaskFlowEnum.RUNNING.getMsg());
                            dingUtils.send(error);
                            timedCache.put(taskInfo, "1");
                        }
                    }
                }
            }

            log.info("\nawait task:{}\ntotal task:{}\nsuccess task:{}\nfailure task:{}\n"
                    , awaitTask
                    , totalTask
                    , successTask
                    , failedTask
            );

            if (flag) {
                break;
            }
            Thread.sleep(10000);
        }
        log.info("end");
        return failureTask;
    }
}
