package com.winhc.dataworks.flow.touch;

import com.helospark.lightdi.LightDi;
import com.helospark.lightdi.LightDiContext;
import com.helospark.lightdi.annotation.Autowired;
import com.helospark.lightdi.annotation.Service;
import com.winhc.dataworks.flow.touch.service.QueryService;
import com.winhc.dataworks.flow.touch.utils.DingUtils;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;

import java.time.LocalTime;
import java.time.temporal.ChronoUnit;

/**
 * @Author: XuJiakai
 * @Date: 2020/8/7 13:41
 * @Description:
 */
@Slf4j
@Service
public class QueryStatus {

    public static void main(String[] args) {
        LightDiContext context = LightDi.initContextByPackage(Main.class.getPackage().getName());
        QueryStatus bean = context.getBean(QueryStatus.class);
        bean.waitTaskDone();
    }


    @Autowired
    private QueryService queryService;

    @Autowired
    private DingUtils dingUtils;

    @SneakyThrows
    public void waitTaskDone() {
        boolean flag = false;
        LocalTime startTime = LocalTime.now();

        LocalTime sendTime = LocalTime.of(8, 0, 0);
        LocalTime exitTime = LocalTime.of(10, 0, 0);

        while (!queryService.isSuccess()) {
            log.warn("等待上游数据同步...");

            if (LocalTime.now().isAfter(sendTime) && !flag) {
                flag = true;
                log.error("上游数据同步任务长时间阻塞，请及时排查！");
                dingUtils.send("上游数据同步任务长时间阻塞，请及时排查！");
            }

            if (LocalTime.now().isAfter(exitTime)) {
                log.error("数据同步任务长时间未完成，程序退出！");
                dingUtils.send("数据同步任务长时间未完成，程序退出！");
                System.exit(-99);
            }
            Thread.sleep(120000);
        }
        long between = ChronoUnit.MINUTES.between(startTime, LocalTime.now());
        String msg = "上游数据同步任务已经完成！累计阻塞" + between + "分钟";
        dingUtils.send(msg);
        log.info(msg);
    }

}
