package com.winhc.dataworks.flow.touch;

import com.aliyuncs.dataworks_public.model.v20180601.CreateManualDagResponse;
import com.helospark.lightdi.LightDi;
import com.helospark.lightdi.LightDiContext;
import com.helospark.lightdi.annotation.Autowired;
import com.helospark.lightdi.annotation.Service;
import com.winhc.dataworks.flow.touch.bean.DataWorksFlowJob;
import com.winhc.dataworks.flow.touch.bean.DataWorksFlowTask;
import com.winhc.dataworks.flow.touch.bean.TaskFlowEnum;
import com.winhc.dataworks.flow.touch.bean.TaskParam;
import com.winhc.dataworks.flow.touch.configuration.SchemaInit;
import com.winhc.dataworks.flow.touch.service.TouchService;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;

import java.util.List;
import java.util.Map;

/**
 * @Author: XuJiakai
 * @Date: 2020/6/23 19:27
 * @Description:
 */
@Slf4j
@Service
public class SingleJobMain {
    @Autowired
    TouchService touchService;
    private static final String file = "D:\\code\\IdeaProjects\\DataWorks-flow-touch\\jobs\\task-step02.yaml";

    @SneakyThrows
    public static void main(String[] args) {
        if (args.length != 3) {
            System.out.println("请输入调度的：业务流程名 任务名 bizDate");
            System.exit(-9999);
        }
        LightDiContext context = LightDi.initContextByPackage(SingleJobMain.class.getPackage().getName());
        SingleJobMain bean = context.getBean(SingleJobMain.class);
        List<DataWorksFlowJob> jobs = SchemaInit.getJobs(file);
        bean.start(args[0], args[1], args[2], jobs);
    }

    @SneakyThrows
    public void start(String flow, String taskName, String bizDate, List<DataWorksFlowJob> jobs) {
        DataWorksFlowJob dataWorksFlowJob = jobs.stream().filter(j -> j.getFlow().equals(flow)).findFirst().orElseThrow(NullPointerException::new);

        DataWorksFlowTask dataWorksFlowTask = dataWorksFlowJob.getTask().stream().filter(t -> t.getTaskName().equals(taskName)).findFirst().orElseThrow(NullPointerException::new);

        TaskParam build = TaskParam.builder()
                .projectName(dataWorksFlowJob.getProject())
                .bizDate(bizDate)
                .flowName(dataWorksFlowJob.getFlow())
                .nodeParam(dataWorksFlowTask.toNodeParam(bizDate))
                .build();
        CreateManualDagResponse touch = touchService.touch(build);
        if(touch == null){
            return;
        }
        while (true) {
            Map<String, TaskFlowEnum> query = touchService.query(dataWorksFlowJob.getProject(), touch.getReturnValue());
            long count = query.values()
                    .stream()
                    .filter(e -> !(TaskFlowEnum.SUCCESS.equals(e) || TaskFlowEnum.FAILURE.equals(e)))
                    .count();

            if (count != 0) {
                Thread.sleep(10000);
            } else {
                log.info("end");
                return;
            }
        }
    }
}
