package com.winhc.dataworks.flow.touch.cmd;

import cn.hutool.core.io.file.FileWriter;
import cn.hutool.core.util.RuntimeUtil;
import com.winhc.dataworks.flow.touch.bean.SparkJobInfo;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.SystemUtils;

import java.io.File;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Objects;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * @Author: XuJiakai
 * @Date: 2020/8/5 15:26
 * @Description:
 */
@Slf4j
public class OdpsCmd {
    private String project_name;
    private String access_id;
    private String access_key;
    private String basePath;
    private String baseCmd;

    public OdpsCmd(String project_name, String access_id, String access_key, String basePath) {
        this.project_name = project_name;
        this.access_id = access_id;
        this.access_key = access_key;
        this.basePath = basePath;
        this.baseCmd = basePath + File.separator + "bin" + File.separator + "odpscmd";
        this.baseCmd += SystemUtils.IS_OS_WINDOWS ? ".bat" : "";
        this.writeConfig(new OdpsConfg(project_name, access_id, access_key).toString());
    }

    @Getter
    @Setter
    @AllArgsConstructor
    private static class OdpsConfg {
        private String project_name;
        private String access_id;
        private String access_key;

        @Override
        public String toString() {
            return "project_name=" + project_name + "\n" +
                    "access_id=" + access_id + "\n" +
                    "access_key=" + access_key + "\n" +
                    "end_point=http://service.odps.aliyun.com/api\n" +
                    "log_view_host=http://logview.odps.aliyun.com\n" +
                    "https_check=true\n" +
                    "data_size_confirm=100.0\n" +
                    "update_url=http://repo.aliyun.com/odpscmd\n" +
                    "use_instance_tunnel=true\n" +
                    "instance_tunnel_max_record=10000";
        }
    }

    private void writeConfig(String str) {
        FileWriter writer = new FileWriter(basePath + File.separator + "conf" + File.separator + "odps_config.ini");
        writer.write(str);
    }


    private synchronized List<String> execute(String cmd) {
        String c = baseCmd + " --project winhc_eci_dev " + " -e \"" + cmd + "\";";
        log.info(c);
        List<String> execute = new ArrayList<>();
        if (SystemUtils.IS_OS_WINDOWS) {
            execute = RuntimeUtil.execForLines(c);
        } else {
            String fs = basePath + "/xjk_start.sh";
            try {
                File file = new File(fs);
                if (!file.exists()) {
                    file.createNewFile();
                }
                file.setExecutable(true);
                List<String> list = new ArrayList<String>();
                list.add("#!/bin/bash");
                list.add(c);
                FileUtils.writeLines(file, list, false);
                execute = RuntimeUtil.execForLines(fs);
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            }
        }
        log.info("out:\n{}", String.join("\n", execute));
        return execute;
    }

    private static final Pattern pattern = Pattern.compile("(\\d{4}-\\d{2}-\\d{2} \\d{2}:\\d{2}:\\d{2}) +(\\w+?) +(\\w+?) +(\\w+?) +([0-9a-zA-Z\\._]+?) *");

    public Boolean kill(String instanceId) {
        execute("spark kill -i " + instanceId);
        return true;
    }

    public List<SparkJobInfo> query(String state) {
        List<String> execute = execute("spark list -s " + state.toUpperCase());
        return execute.stream().filter(str -> !str.startsWith("StartTime")).map(str -> {
            Matcher m = pattern.matcher(str);
            if (m.find()) {
                return new SparkJobInfo(parse(m.group(1)), m.group(2), m.group(3), m.group(4), m.group(5));
            } else {
                log.error("error str : {}", str);
                return null;
            }
        }).filter(Objects::nonNull).collect(Collectors.toList());
    }

    DateTimeFormatter dtf = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")
            .withLocale(Locale.CHINA)
            .withZone(ZoneId.systemDefault());

    private LocalDateTime parse(String str) {
        LocalDateTime parse = LocalDateTime.parse(str, dtf);
        return parse;
    }
}
