package com.winhc.dataworks.flow.touch.configuration;

import lombok.extern.slf4j.Slf4j;
import okhttp3.Interceptor;
import okhttp3.Request;
import okhttp3.Response;

import java.io.IOException;

/**
 * @Author: XuJiakai
 * @Date: 2020/6/28 14:38
 * @Description:
 */
@Slf4j
public class OkHttpRetryInterceptor implements Interceptor {
    //最大重试次数
    public int maxRetry;
    //假如设置为3次重试的话，则最大可能请求4次（默认1次+3次重试）

    public OkHttpRetryInterceptor(int maxRetry) {
        this.maxRetry = maxRetry;
    }

    @Override
    public Response intercept(Chain chain) throws IOException {
        Request request = chain.request();
        int retryNum = 0;
        while (true) {
            try {
                Response response = chain.proceed(request);
                while (!response.isSuccessful() && retryNum < maxRetry) {
                    retryNum++;
                    log.error("response code : {} ok http retry:{} request url: {} ,response body:{}", response.code(), retryNum, request.url(), response.body().string());
                    response.close();
                    response = chain.proceed(request);
                }
                return response;
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                log.error("ok http retry:{} request url: {} ", retryNum, request.url());
                if (retryNum++ > maxRetry) {
                    throw e;
                }
            }
        }
    }
}
