package com.winhc.dataworks.flow.touch.utils;

import com.winhc.dataworks.flow.touch.bean.SparkJobInfo;
import com.winhc.dataworks.flow.touch.cmd.OdpsCmd;
import lombok.extern.slf4j.Slf4j;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @Author: XuJiakai
 * @Date: 2020/8/17 10:26
 * @Description:
 */
@Slf4j
public class SparkDaemonThread extends Thread {
    private OdpsCmd odpsCmd;
    private long maxMinutes;
    private long interval;

    /**
     * @param project_name    工程名
     * @param access_id       key
     * @param access_key      secret
     * @param basePath        odps根目录
     * @param maxMinutes      超时间 分钟
     * @param intervalMinutes 守护间隔时间，默认为10分钟
     */
    public SparkDaemonThread(String project_name, String access_id, String access_key, String basePath, long maxMinutes, double intervalMinutes) {
        super.setDaemon(true);
        super.setName("spark-daemon");
        this.maxMinutes = maxMinutes;
        this.interval = (long) (intervalMinutes * 60 * 1000);
        this.odpsCmd = new OdpsCmd(project_name, access_id, access_key, basePath);
    }

    /**
     * @param project_name 工程名
     * @param access_id    key
     * @param access_key   secret
     * @param basePath     odps根目录
     * @param maxMinutes   超时间 分钟
     */
    public SparkDaemonThread(String project_name, String access_id, String access_key, String basePath, long maxMinutes) {
        this(project_name, access_id, access_key, basePath, maxMinutes, 5L);
    }


    @Override
    public void run() {
        try {
            while (true) {
                List<SparkJobInfo> running = odpsCmd.query("RUNNING");
                List<String> timeOut = running.stream().filter(e -> e.runningMinutes() >= maxMinutes).map(SparkJobInfo::getInstanceId).collect(Collectors.toList());
                if (!timeOut.isEmpty()) {
                    for (String s : timeOut) {
                        log.warn("kill InstanceId : {}", s);
                        odpsCmd.kill(s);
                    }
                }
                Thread.sleep(interval);
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }
}
